--[[
  Premo Power version 1.6.1 beta
  Unit Power movable bar for Paladins, Priests, Rogues and Druids
  By Azmaedus 
  5.0.4 compatible
  Special thanks to Hoern (ristretto Power) 
--]]

local pinfo = {
  class = string.upper( select( 2, UnitClass( 'player' ) ) ),
  level = UnitLevel("player"),
}

local BackDrop = {
		bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
    edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]],
    tile = true, tileSize = 4, edgeSize = 4,
    insets = { left = 1, right = 1, top = 1, bottom = 1 }
}

local BackDropAlpha = 0.3
local PALADIN_MAX_BLIP = 5
local SPRIEST_MAX_BLIP = 3
local DPRIEST_MAX_BLIP = 5
local PRIEST_SHADOW_SPEC = 3
local PRIEST_DISC_SPEC = 1
local sliders_loaded = false

--LUA SavedVars ================================================================

if PremoPowerCfg == nil then
	PremoPowerCfg = {
		align = "CENTER",
		x   = 0,
		y   = 0,
		red = 200,
		green = 200,
		blue = 100,
		scale = 1,
		enabled = true,
		w = 15,
		h = 15,
	}
end

-- local sliders_loaded = false

--Main Frame ===================================================================

PremoPowerFrame = CreateFrame( "Frame", "PremoPowerFrame" )
PremoPowerFrame:RegisterEvent( "VARIABLES_LOADED" )

PremoPowerFrame:SetScript( 
  "OnEvent", 
  function( self, event, ... )
  	if event == "VARIABLES_LOADED" then
  		self:UnregisterEvent( "VARIABLES_LOADED" )
  		self:Init()
  	end
  end )
  
--Functions ====================================================================

function PremoPowerFrame:Init()
  UpdateTexture( RGB_DisplayFull, PremoPowerCfg.red, PremoPowerCfg.green, PremoPowerCfg.blue, 1 );
  UpdateTexture( RGB_DisplayEmpty, PremoPowerCfg.red, PremoPowerCfg.green, PremoPowerCfg.blue, BackDropAlpha );
  
  sliders_loaded = true
  
  RGB_Red_Slider:SetValue( PremoPowerCfg.red );
  RGB_Green_Slider:SetValue( PremoPowerCfg.green );
  RGB_Blue_Slider:SetValue( PremoPowerCfg.blue );

-- PALADIN - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  if pinfo.class == "PALADIN" then
		local red = PremoPowerCfg.red / 256
		local green = PremoPowerCfg.green / 256
		local blue = PremoPowerCfg.blue / 256
		makeFrames( PALADIN_MAX_BLIP, red, green, blue, PremoPowerCfg.w, PremoPowerCfg.h )
		updateFrames( PALADIN_MAX_BLIP, get_HolyPower(), red, green, blue )
		PremoPowerFrame:RegisterEvent( "UNIT_POWER" )
		
    PremoPowerFrame:SetScript( 
      "OnEvent", 
      function( self, event, unit, power )
    		local red = PremoPowerCfg.red /256
    		local green = PremoPowerCfg.green / 256
    		local blue = PremoPowerCfg.blue / 256
        if unit ~= "player" or power ~= "HOLY_POWER" then
          return
        end
        updateFrames( PALADIN_MAX_BLIP, get_HolyPower(), red, green, blue )
		  end )
	end
	
-- PRIEST - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	if pinfo.class == "PRIEST" then
	  local spec = GetSpecialization() 
    if ( spec == PRIEST_SHADOW_SPEC and pinfo.level > 20 ) or
       ( spec == PRIEST_DISC_SPEC and pinfo.level > 49 ) then 
      local red = PremoPowerCfg.red /256
		  local green = PremoPowerCfg.green / 256
		  local blue = PremoPowerCfg.blue / 256

      if spec == PRIEST_SHADOW_SPEC then
        makeFrames( SPRIEST_MAX_BLIP, red, green, blue, PremoPowerCfg.w, PremoPowerCfg.h )
        updateFrames( SPRIEST_MAX_BLIP, get_ShadowOrbs(), red, green, blue )
      elseif spec == PRIEST_DISC_SPEC then
        makeFrames( DPRIEST_MAX_BLIP, red, green, blue, PremoPowerCfg.w, PremoPowerCfg.h )
        updateFrames( DPRIEST_MAX_BLIP, get_EvangelismStacks(), red, green, blue )
      end
      
		  PremoPowerFrame:RegisterEvent( "UNIT_AURA" )
		  PremoPowerFrame:RegisterEvent( "ACTIVE_TALENT_GROUP_CHANGED" )
		  PremoPowerFrame:SetScript( 
        "OnEvent", 
      
        function( self, event, unit )

      	 local red = PremoPowerCfg.red /256
      	 local green = PremoPowerCfg.green / 256
      	 local blue = PremoPowerCfg.blue / 256

			   if event == "UNIT_AURA" then
				   if unit ~= "player" then
             return
           end
				  
           if spec == PRIEST_SHADOW_SPEC then
             updateFrames( SPRIEST_MAX_BLIP, get_ShadowOrbs(), red, green, blue )
           elseif spec == PRIEST_DISC_SPEC then
             updateFrames( DPRIEST_MAX_BLIP, get_EvangelismStacks(), red, green, blue )
           end

        elseif event == "ACTIVE_TALENT_GROUP_CHANGED" then
				  if spec == PRIEST_SHADOW_SPEC then
				    PremoPowerFrame:RegisterEvent( "UNIT_AURA" )
				    setVisible( true )
				    updateFrames( SPRIEST_MAX_BLIP, get_ShadowOrbs(), red, green, blue )
				  elseif spec == PRIEST_DISC_SPEC then
				    PremoPowerFrame:RegisterEvent( "UNIT_AURA" )
				    setVisible( true )
				    updateFrames( DPRIEST_MAX_BLIP, get_EvangelismStacks(), red, green, blue )
				  else
				    PremoPowerFrame:UnregisterEvent( "UNIT_AURA" )
				    setVisible( false )
				  end
			  end
		  end )
    end
  end
  
end --Init()

--------------------------------------------------------------------------------

function getMaxFrames()
  if pinfo.class == "PALADIN" then
    return PALADIN_MAX_BLIP
  elseif pinfo.class == "PRIEST" then
    local spec = GetSpecialization()
    
    if spec == PRIEST_SHADOW_SPEC then
      return SPRIEST_MAX_BLIP
    elseif spec == PRIEST_DISC_SPEC then 
      return DPRIEST_MAX_BLIP
    end
  end
end

--------------------------------------------------------------------------------

function genFrame( red, green, blue, w, h )
	local f = CreateFrame( "Frame" )
	f:SetWidth( w )
	f:SetHeight( h )
	f:SetBackdrop( BackDrop )
	f:SetBackdropColor( red, green, blue, BackDropAlpha )
	f:SetBackdropBorderColor( 0, 0, 0, 1 )
	f:Show()
	return f
end

--------------------------------------------------------------------------------

function makeFrames( num, red, green, blue, w, h )
	for i = 1, num do
		_G['powerframe'..i] = genFrame( red, green, blue, w, h )
		if i == 1 then
			_G['powerframe1']:SetPoint( PremoPowerCfg.align, UIParent, PremoPowerCfg.align, PremoPowerCfg.x, PremoPowerCfg.y )
			_G['powerframe1']:SetMovable( true )
			_G['powerframe1']:EnableMouse( true )
			_G['powerframe1']:RegisterForDrag( "LeftButton" )
			_G['powerframe1']:SetScript( "OnDragStart", function( self )
				if IsAltKeyDown() then
					self:StartMoving()
				end
			end )
			
      _G['powerframe1']:SetScript( "OnDragStop", function( self )
				self:StopMovingOrSizing()
				_,_, PremoPowerCfg.align, PremoPowerCfg.x, PremoPowerCfg.y = self:GetPoint( 1 )
			end )
			
      _G['powerframe1']:SetScale( PremoPowerCfg.scale )
		else
			_G['powerframe'..i]:SetPoint( "TOPLEFT", _G['powerframe'..i - 1], "TOPRIGHT", 4, 0 )
			_G['powerframe'..i]:SetScale( PremoPowerCfg.scale )
		end
	end
end

--------------------------------------------------------------------------------

function get_HolyPower()
	return UnitPower( "player", SPELL_POWER_HOLY_POWER )
end

--------------------------------------------------------------------------------

function get_ShadowOrbs()
	return UnitPower( "player", SPELL_POWER_SHADOW_ORBS )
end

--------------------------------------------------------------------------------

function get_EvangelismStacks()
	local _, _, _, count = UnitAura( "player", "Evangelism" ) --81661
	if count == nil then
    return 0
  else
    return count
  end
end

--------------------------------------------------------------------------------

function get_ComboPoints()
  return GetComboPoints("player" ,"target")
end

--------------------------------------------------------------------------------

function setVisible( visible )
	if visible then
		for i = 1, getMaxFrames() do
			_G['powerframe'..i]:Show()
		end
	else
		for i = 1, getMaxFrames() do
			_G['powerframe'..i]:Hide()
		end
	end
end

--------------------------------------------------------------------------------

function updateFrames( max, cur, red, green, blue )
	for i = 1, max do
		if i <= cur then
			_G['powerframe'..i]:SetBackdropColor( red, green, blue, 1 )
		else
			_G['powerframe'..i]:SetBackdropColor( red, green, blue, BackDropAlpha )
		end
	end
end

--------------------------------------------------------------------------------

function UpdateTexture( tex, r, g, b, a )
  tex:SetTexture( r / 256, g / 256, b / 256, a )
end

--------------------------------------------------------------------------------

function UpdateScale( s )
	for i = 1, getMaxFrames() do
		_G['powerframe'..i]:SetScale( s )
	end
end

--------------------------------------------------------------------------------

function UpdatePosition()
  _G['powerframe1']:SetPoint( PremoPowerCfg.align, UIParent, PremoPowerCfg.align, PremoPowerCfg.x, PremoPowerCfg.y )      
end

--------------------------------------------------------------------------------

function UpdateDimensions()
  for i = 1, getMaxFrames() do
	  _G['powerframe'..i]:SetHeight( PremoPowerCfg.h )
    _G['powerframe'..i]:SetWidth( PremoPowerCfg.w )
  end      
end

--Slash Commands ===============================================================

SLASH_PREMO_POWER1 = "/ppower"
SlashCmdList["PREMO_POWER"] = function(str)
	local switch, message = str:match("^(%S*)%s*(.-)$");
	local cmd = string.lower( switch )
	local msg = string.lower( message )

  if cmd == "options" then
    PPowerOptions:Show();

  elseif cmd == "w" then
    if tonumber(msg) ~= nil then
      PremoPowerCfg.w = msg
      UpdateDimensions()
    end;    

  elseif cmd == "h" then
    if tonumber(msg) ~= nil then
      PremoPowerCfg.h = msg
      UpdateDimensions()
    end;

  elseif cmd == "show" then
    for i = 1, getMaxFrames() do
		  _G['powerframe'..i]:Show();
    end
    print(PPowerLocal.Show)

  elseif cmd == "hide" then
    for i = 1, getMaxFrames() do
		  _G['powerframe'..i]:Hide();
    end  
    print(PPowerLocal.Hide)

  elseif cmd == "x" then
    if tonumber(msg) ~= nil then
      PremoPowerCfg.x = msg
      UpdatePosition()
    end;    

  elseif cmd == "y" then
    if tonumber(msg) ~= nil then
      PremoPowerCfg.y = msg
      UpdatePosition()
    end;    
  end
end

--------------------------------------------------------------------------------
